BuffDrop = {}
local core = BuffDrop

local frame = CreateFrame("Frame")
frame:RegisterEvent("PLAYER_ENTERING_WORLD")
frame:SetScript("OnEvent", function(this, event, ...)
    core[event](...)
end)

--Respond to events:
function core.PLAYER_ENTERING_WORLD()
	local function setup_db(db, defaults)
		for k,v in pairs(defaults) do
			if not db[k] then
				db[k] = v
			end
		end
		return db
	end
	if not BuffDropDB then BuffDropDB = {} end
	if not BuffDropPCDB then BuffDropPCDB = {} end
	core.db = setup_db(BuffDropDB, {
		icon = true,
		raid = true,
		icontime = 2,
	})
	core.chardb = setup_db(BuffDropPCDB, {
		watch = {},
	})
	frame:UnregisterEvent("PLAYER_ENTERING_WORLD")
	
	frame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
end

function core.COMBAT_LOG_EVENT_UNFILTERED(timestamp, eventtype, sourceguid, sourcename, sourceflags, destguid, destname, destflags, ...)
	if core[eventtype] and CombatLog_Object_IsA(destflags, COMBATLOG_FILTER_MINE) then
		core[eventtype](...)
	end
end

function core.SPELL_AURA_REMOVED(spellid, spellname, spellschool, auratype)
	if core.chardb.watch[spellname] then
		core.AnnounceBuff(spellname)
	end
end

function core.SPELL_AURA_APPLIED(spellid, spellname, spellschool, auratype)
	if core.iconbuff == spellname then
		core.HideBigIcon()
	end
end

local function get_buff_icon(buff)
	for i=1, BUFF_MAX_DISPLAY do
		local n, _, t = UnitAura("player", i)
		if not n then break end
		if n == buff then return t end
	end
end
local color_red = {r=1, g=0, b=0}
function core.AnnounceBuff(buff)
	if core.db.raid then
		RaidNotice_AddMessage(RaidWarningFrame, buff, color_red)
	end
	if core.db.icon then
		local icon = get_buff_icon(buff)
		if icon then
			core.iconbuff = buff
			core.ShowBigIcon(icon)
		end
	end
end

local bigicon
local bigicon_t = 0
function core.ShowBigIcon(icon)
	if not bigicon then
		frame:SetFrameStrata("HIGH")
		frame:SetPoint("CENTER", WorldFrame)
		local width = WorldFrame:GetHeight() / 3
		frame:SetWidth(width)
		frame:SetHeight(width)
		
		bigicon = frame:CreateTexture()
		bigicon:SetAllPoints(frame)
		bigicon:SetBlendMode("BLEND")
		bigicon:SetTexCoord(0.09, 0.91, 0.09, 0.91)
		
		frame:SetScript("OnUpdate", function()
			bigicon_t = bigicon_t + arg1
			bigicon:SetVertexColor(1, 1, 1, 0.5 - (0.5 * (bigicon_t / core.db.icontime)))
			if bigicon_t > core.db.icontime then
				core.HideBigIcon()
			end
		end)
		frame:Hide()
	end
	bigicon:SetVertexColor(1, 1, 1, 0.5)
	bigicon:SetTexture(icon)
	frame:Show()
end
function core.HideBigIcon()
	core.iconbuff = nil
	bigicon:SetTexture(1, 0, 0)
	frame:Hide()
	bigicon_t = 0
end

local function announce(message, r, g, b)
	DEFAULT_CHAT_FRAME:AddMessage(message, r, g, b)
end
local menu_options = {
	icon = function(arg)
		if not arg then return " - big icon (" .. (core.db.icon and "true" or "false") .. ")" end
		core.db.icon = not core.db.icon
		announce("Huge icon "..(core.db.icon and "enabled" or "disabled"), 1, 1, 1)
	end,
	raid = function(arg)
		if not arg then return " - raid warning (" .. (core.db.raid and "true" or "false") .. ")" end
		core.db.raid = not core.db.raid
		announce("Raid warning "..(core.db.raid and "enabled" or "disabled"), 1, 1, 1)
	end,
	watch = function(arg)
		if not arg then return " - buffs to watch" end
		if arg == "" then
			announce("Watching:", 1, 1, 1)
			for buff in pairs(core.chardb.watch) do
				announce(" "..buff, 1, 1, 1)
			end
			return
		end
		if core.chardb.watch[arg] then
			core.chardb.watch[arg] = nil
			announce("No longer watching "..arg, 1, 1, 1)
		else
			core.chardb.watch[arg] = true
			announce("Now watching "..arg, 1, 1, 1)
		end
	end,
	icontime = function(arg)
		if not arg then return " - icon lingers for (" .. core.db.icontime .. ")" end
		local newtime = tonumber(arg)
		if newtime then
			core.db.icontime = newtime
		end
	end,
}
SLASH_BUFFDROP1 = "/buffdrop"
SLASH_BUFFDROP2 = "/bd"
SlashCmdList["BUFFDROP"] = function(arg)
	local command, arguments = string.match(arg, "^(%a+) ?(.*)")
	if menu_options[command] then
		return menu_options[command](arguments)
	end
	announce("BuffDrop:", 1, 1, 1)
	for k,v in pairs(menu_options) do
		announce(" -"..k..v(), 1, 1, 1)
	end
end
